from django.contrib.sites.models import Site
from django.db import models
from django.conf import settings
from garpix_page.utils.all_sites import get_all_sites
from garpix_utils.file import get_file_path
from mptt.models import MPTTModel, TreeForeignKey
from garpix_utils.managers import ActiveManager
from ..mixins import LinkMixin
from ..utils import check_is_home
from ..validators import validate_type, validate_size
from garpix_utils.managers import GCurrentSiteManager
from django.utils.translation import ugettext_lazy as _


class MenuItem(LinkMixin, MPTTModel):
    """
    Пункты меню.
    """
    title_for_admin = models.CharField(max_length=100, blank=True, default='', verbose_name=_('Название для админа'))
    title = models.CharField(max_length=100, verbose_name=_('Название'))
    icon = models.FileField(
        upload_to=get_file_path, verbose_name=_('Иконка'), blank=True, null=True,
        validators=[validate_type, validate_size]
    )
    menu_type = models.CharField(default='', max_length=100, choices=settings.CHOICE_MENU_TYPES,
                                 verbose_name=_('Тип меню'))
    is_active = models.BooleanField(default=True, verbose_name=_('Включено'))
    target_blank = models.BooleanField(default=False, verbose_name=_('Открывать в новом окне'))
    created_at = models.DateTimeField(auto_now_add=True, verbose_name=_('Дата создания'))
    updated_at = models.DateTimeField(auto_now=True, verbose_name=_('Дата изменения'))
    sort = models.IntegerField(default=100, verbose_name=_('Сортировка'),
                               help_text=_('Чем меньше число, тем выше будет элемент в списке.'))
    parent = TreeForeignKey('self', null=True, blank=True, related_name='children', db_index=True,
                            verbose_name=_('Родительский пункт меню'), on_delete=models.CASCADE)
    css_class = models.CharField(max_length=100, null=True, blank=True,
                                 help_text=_('Можно задать, если пункту меню необходима особенная стилизация'),
                                 verbose_name=_('CSS класс'))
    sites = models.ManyToManyField(Site, default=get_all_sites, verbose_name=_('Сайты для отображения'))
    is_current = False
    is_current_full = False
    active_manager = ActiveManager()
    on_site = GCurrentSiteManager()

    class Meta:
        verbose_name = 'Пункт меню | Menu Point'
        verbose_name_plural = 'Пункты меню | Menu Points'
        ordering = ('sort',)
        abstract = False

    def __str__(self):
        if self.title_for_admin:
            return self.title_for_admin
        return self.title

    def get_menu_params(self):
        return settings.MENU_TYPES[self.menu_type]

    def get_active_children(self):
        return MenuItem.on_site.filter(parent=self, is_active=True)

    def get_is_current(self, current_path):

        link = self.get_link()

        current_path_without_slash = current_path
        if current_path_without_slash[-1] == '/':
            current_path_without_slash = current_path_without_slash[0:-1]

        if link in (current_path, current_path_without_slash):
            return True
        elif current_path.startswith(link):
            if not check_is_home(link):
                return True
        elif self.url and self.url.endswith(current_path):
            if not check_is_home(link):
                return True

        return False

    def get_is_current_full(self, current_path):

        current_path_without_slash = current_path
        if current_path_without_slash[-1] == '/':
            current_path_without_slash = current_path_without_slash[0:-1]

        return self.get_link() in (current_path, current_path_without_slash)



from rest_framework.fields import SerializerMethodField
from rest_framework.serializers import ModelSerializer
from .models import MenuItem


class MenuItemSerializer(ModelSerializer):
    link = SerializerMethodField()
    is_current = SerializerMethodField()
    is_current_full = SerializerMethodField()

    def get_link(self, instance):
        request = self.context.get('request', None)
        return instance.get_link(request)

    def get_is_current(self, instance):
        current_path = self.context.get('current_path', None)

        if current_path:
            return instance.get_is_current(current_path)
        return False

    def get_is_current_full(self, instance):
        current_path = self.context.get('current_path', None)

        if current_path:
            return instance.get_is_current_full(current_path)
        return False

    def get_field_names(self, declared_fields, info):
        expanded_fields = super().get_field_names(declared_fields, info)

        if getattr(self.Meta, 'extra_fields', None):
            return expanded_fields + self.Meta.extra_fields
        else:
            return expanded_fields

    class Meta:
        model = MenuItem
        exclude = (
            'title_for_admin',
            'subpage_url',
            'sites',
            'page',
            'lft',
            'rght',
            'tree_id',
            'level',
            'created_at',
            'updated_at',
        )
        extra_fields = ['link', 'is_current', 'is_current_full']


class MenuItemWithChildrenSerializer(MenuItemSerializer):
    children = SerializerMethodField()

    def get_children(self, instance):
        items = instance.get_active_children()
        return MenuItemWithChildrenSerializer(items, many=True, context=self.context).data

    class Meta(MenuItemSerializer.Meta):
        extra_fields = MenuItemSerializer.Meta.extra_fields + ['children']



from django.conf import settings
from django.forms.models import model_to_dict


def check_is_home(url):
    urls = ['', '/']
    for item in settings.LANGUAGES:
        urls.append(f'/{item[0]}')
    return url in urls


def get_menu_item_dict(menu_item, current_path):
    context = model_to_dict(menu_item)
    link = menu_item.get_link()

    context['get_link'] = link
    context['is_current'] = menu_item.get_is_current(current_path)
    context['is_current_full'] = menu_item.get_is_current_full(current_path)
    context['object'] = menu_item
    try:
        context['icon'] = menu_item.icon.url
    except Exception:
        context['icon'] = None

    context.pop('page', None)
    context.pop('title_for_admin', None)

    return context


def get_menus(current_path):
    from garpix_menu.models import MenuItem

    menus = {}
    menu_items = MenuItem.on_site.prefetch_related('sites').filter(is_active=True, parent=None).order_by('sort', 'title')
    for menu_type_arr in settings.CHOICE_MENU_TYPES:
        menu = list(filter(lambda item: item.menu_type == menu_type_arr[0], menu_items))
        menus[menu_type_arr[0]] = []
        for menu_item in menu:
            menus[menu_type_arr[0]].append(get_menu_item_dict(menu_item, current_path))

    return menus



from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.viewsets import ModelViewSet
from rest_framework.permissions import IsAdminUser
from .models import MenuItem
from .serializers import MenuItemSerializer


class MenuItemViewSet(ModelViewSet):
    queryset = MenuItem.on_site.all()
    serializer_class = MenuItemSerializer
    permission_classes = [IsAdminUser, ]
    filter_backends = (DjangoFilterBackend, )
    filter_fields = {
        'title': ['icontains', ],
    }
